## January 30, 2018

## Danica Schaffer-Smith
## danica.schaffer.smith@duke.edu

## Code used to produce Figure 7 in Schaffer-Smith et al. 2018. 
## Quantifying shorebird habitat in managed wetlands by modeling 
## shallow water depth dynamics. Ecological Applications. 

## We modelled historical water depth at the peak of spring migration (early April)
## using Landsat-derived water extent maps (30-m) in combination with 
## high-resolution (5-m) topography mapped in the field. 

## Script steps
# 1) Create a time series stack of water maps for late March - late April
# 2) Read in a the DEM for example wetland Sacramento National Wildlife Refuge Tract 41, cell 1 (SAC:T41:1)
# 3) Loop through the water map time series stack, and for each image date:
      # a) Resample and snap to high-resolution DEM
      # b) Extract the DEM by the water map and calculate the maximum elevation cell within the water class. 
          # Water maps have three values (0 = nonwater, 1 = water, 2 = NoData/Clouds or Urban)
      # c) Tabulate the number of nonwater, water, and NA cells in the wetland

## The water maps used in the third step are available from a different section of the Duke Digital Repository: 
## "water_extent > single_date_water_extent_maps" and are organized into subdirectories:
## "L5" = water maps based on Landsat 4&5 images
## "L7" = water maps based on Landat 7 images
## "L8" = water maps based on Landsat 8 images
## 
## To successfully run the code, copy those directories into the data folder for this R project. 
## 
## Refer to  Schaffer-Smith et al. 2017. for information about methods for water extent mapping. 
## 
## Attribution for use of water extent maps:
## Schaffer-Smith et al. 2017. Three decades of Landsat-derived spring surface water 
## dynamics in an agricultural wetland mosaic; Implications for migratory shorebirds. 
## Remote Sensing of Environment

## The final output from this code should be [date, wetland ID, count nonwater, count water, 
## count NA, water max elevation]
## The final result can be filtered to include the best date ([a]fewest NA pixels, 
## [b]prefer early April, then late March or late April)


## Load required packages
library(ggplot2)
library(maptools)
library(raster)
library(rasterVis)
library(spatial.tools) # For clipping, resampling, aligning rasters


##################################################################
## Example Wetland 1 SAC:T41:1
##################################################################
## Read in the polygon shapefile for the wetland
poly <- readOGR("data/SacT41c1boundary_floodable.shp")

## Read in the wetland DEM
dem <- raster("data/sact41c1_elev_m.tif")

## Create a DEM that is remapped so that all values = 1 (for cell counts by zone later)
dem.max <- cellStats(dem, 'max')
m <- c(0, dem.max, 1)
dem.1 <- reclassify(dem, m)

## Identify the directory containing water extent maps and make a list of water map files
## Replace the root directory with the path where you have stored water extent maps. 
## This must be run for each Landsat directory to create a final stack for the full time serires of water extent
dir <- "PATH/L5/"
#dir <- "PATH/L7/"
#dir <- "PATH/L8/"
water.list <- list.files(dir, pattern=glob2rx('*.img$'))

## Check the number of files in the raster list (grids)
length <- length(water.list) # 150 Landsat 4/5 files
#create a raster stack
s <- stack(paste0(dir, water.list))
N <- nlayers(s) ## This will be the number of loops

## Create a list of the dates for each map
## Modify which line is running to match the Landsat directory
names.list <- names(s)
splitFun <- function(x){
  #split <- strsplit(x, "LT5")
  #split <- strsplit(x, "LE7")
  #split <- strsplit(x, "LC8")
  date <- unlist(split)[2]
  return(date)
}

date.list <- lapply(names.list, splitFun)
date.list <- unlist(date.list)

## Initialize an empty list to store the results of each for loop output
result <- vector("list", N) 

## Extract raster cell count (sum) within each polygon area (poly)
for (i in 1:N) {
  water <- s[[i]]
  water_sync <- spatial_sync_raster(water, dem, method="ngb") # align water map to dem
  max <- zonal(dem, water_sync, fun='max', na.rm=TRUE, df=TRUE)
  count <- zonal(dem.1, water_sync, fun = 'sum', df = T)
  df <- merge(count, max, by = "zone")
  df <- cbind(date.list[i], df)
  result[[i]] <- data.frame(df)
}

result <- do.call(rbind, result)

## Write the results to a CSV file
write.csv(result, file = "data/sact41c1_spring_depth_LT5.csv")
#write.csv(result, file = "data/sact41c1_spring_depth_LE7.csv")
#write.csv(result, file = "data/sact41c1_spring_depth_LC8.csv")

## Combine files into one data frame
lt5 <- read.csv('data/sact41c1_spring_depth_LT5.csv') #Landsat 4/5
le7 <- read.csv('data/sact41c1_spring_depth_LE7.csv') #Landsat 7
lc8 <- read.csv('data/sact41c1_spring_depth_LC8.csv') #Landsat 8

data <- rbind(lt5, le7, lc8)

## Write out the combined data frame
write.csv(data, file = "data/sact41c1_spring_depth_1984-2015.csv")


#################################################################################################################################################################
## A. Data processing/cleanup
#####################################################################################################################
## 1) Calculate the proportion of NA pixels
## 2) Remove records with high proportion of NA pixels
## 3) Limit to early April

## Load packages
library(dplyr)
library(magrittr)
library(lubridate)

data <- read.csv('data/sact41c1_spring_depth_1984-2015.csv')

data <- data %>%
  group_by(date.list.i.) %>%
  mutate(totcells = sum(sum))

pna <- data %>% 
  filter(zone == 2) %>%
  group_by(date.list.i.) %>%
  mutate(pNA = sum/totcells) %>%
  select (date.list.i., pNA)

data <- merge(data, pna, by = "date.list.i.")

data <- data %>%
  filter(zone == 1) %>%
  filter(pNA < 0.1)

data$date <- as.POSIXct(strptime(data$date.list.i., format = '%Y%j'))
data$yday <- as.numeric(format(data$date, "%j"))
data$year <- as.numeric(format(data$date, "%Y"))

## Remove Inf/-Inf values
data <- data[Reduce(`&`, lapply(data, function(x) !is.na(x)  & is.finite(x))),]
str(data) #85 obs


##########################################################
## B. Water depth and habitat distributions over time 
##########################################################
## Produce water depth maps, optimal habitat maps, and calculate proportion of 
## optimal habitat for each date.For each observation in the final dataset we 
## will use the maximum elevation as the water surface height and substract the 
## DEM from the water surface to get depth distributions.

dem.cells <- ncell(dem)

## Convert data to cm
dem <- dem*100 # convert dem to cm
data$max <- data$max*100 # convert the maximum elevation to cm

surface <- !is.na(dem)
surface[surface==1] <- data$max[1]
depth <- surface - dem
m <- c(-Inf, 0, NA, 0, 5, 1, 5, Inf, NA)
depth.5 <- reclassify(depth, m)
plot(depth.5)
p.5cm <- cellStats(depth.5, 'sum')/dem.cells
m3 <- c(-Inf, 0, NA, 0, 10, 1, 10, Inf, NA)
depth.10 <- reclassify(depth, m3)
plot(depth.10)
p.10cm <- cellStats(depth.10, 'sum')/dem.cells
m2 <- c(-Inf, 0, NA, 0, 15, 1, 15, Inf, NA)
depth.15 <- reclassify(depth, m2)
plot(depth.15)
p.15cm <- cellStats(depth.15, 'sum')/dem.cells

#sum <- calc(depth.5cm, 'sum', na.rm=TRUE)
#valid <- sum(!is.na(depth.5cm))
#freq<- (sum/valid)
#plot(freq)


## Initialize empty raster stacks to store the spatial water depth maps, and maps of <5cm, <10cm, and <15cm shallow habitat.
depth <- stack()
depth.5cm <- stack()
depth.10cm <- stack()
depth.15cm <- stack()

## Initialize dataframe to store summary data for proportion of <5cm and <15cm habitat in the wetland
#result <- vector("list", nrow(data)) 

for(i in 1:nrow(data)) { 
  surface <- !is.na(dem)
  maxwater <- data$max[i]
  surface[surface==1] <- maxwater
  x <- surface - dem
  depth <- stack(depth, x)
  
  m <- c(-Inf, 0, 0, 0, 5, 1, 5, Inf, 0)
  depth.5cm <- reclassify(depth, m)
  #depth.5cm <- stack(depth.5cm, y)
  p.5cm <- cellStats(depth.5cm, 'sum')/dem.cells
  
  m3 <- c(-Inf, 0, 0, 0, 10, 1, 10, Inf, 0)
  depth.10cm <- reclassify(depth, m3)
  #depth.5cm <- stack(depth.5cm, y)
  p.10cm <- cellStats(depth.10cm, 'sum')/dem.cells
  
  m2 <- c(-Inf, 0, 0, 0, 15, 1, 15, Inf, 0)
  depth.15cm <- reclassify(depth, m2)
  #depth.15cm <- stack(depth.15cm, z)
  p.15cm <- cellStats(depth.15cm, 'sum')/dem.cells
  
  df <- data.frame(cbind(data$date.list, p.5cm, p.10cm, p.15cm))
}

names(df) <- c("date.list.i.", "p.5cm", "p.10cm", "p.15cm")

## Write out the results
write.csv(df, "data/SacT41c1_phabitat_1984-2015.csv")
writeRaster(depth, "data/SacT41c1_depth_stack.tif", overwrite = T)
writeRaster(depth.5cm, "data/SacT41c1_hab5cm_stack.tif", overwrite = T)
writeRaster(depth.10cm, "data/SacT41c1_hab10cm_stack.tif", overwrite = T)
writeRaster(depth.15cm, "data/SacT41c1_hab15cm_stack.tif", overwrite = T)

depth.10cm.sum <- calc(depth.10cm, sum, na.rm=TRUE)
depth.10cm.valid <- sum(!is.na(depth.10cm))
depth.10cm.freq <- (depth.10cm.sum/depth.10cm.valid)
plot(depth.10cm.freq)

##########################################################################
## C. Peak migration optimal habitat proportions over time
##########################################################################

data <- merge(data, df, by = "date.list.i.")

## Filter to mid-March/early April dates from 1996 -2015
## We limited our historical assessment to 1996 - 2015, because this is the period we were 
## certain that no major topographical changes had taken place in study wetlands. 
peak <- data %>% 
  filter(yday > 73 & yday < 107)%>%
  filter(date > "1996-01-01")
str(peak) # 18 dates with sufficient data available -- remove duplicates with higher pNA

peak <- peak %>% 
  group_by(year) %>%
  slice(which.min(pNA))
str(peak) #14 years

## Add California Department of Water Resources Water Year Index and Water Year Type information
## This data was accessed from: http://cdec.water.ca.gov/cgi-progs/iodir/WSIHIST
## W       Wet year type
## AN      Above normal year type
## BN      Below normal year type
## D       Dry year type
## C       Critical year type
WY <- read.csv("data/CDEC_WYHIST_1-19-16_Sac_1983-2015.csv", header = T)
WY$year <- WY$WY+1
WY <- WY %>% 
  select(WY, Index, Yr.type, year)

peak <- merge(peak, WY, by = "year")
levels(peak$Yr.type) <- c("C", "D", "BN", "AN", "W")
peak$drought <- 0
drought <- c("C", "D", "BN")
peak$drought[which(peak$Yr.type %in% drought)] <- 1
peak$drought <- as.factor(peak$drought)

peak$survey <- "SacT41c1"

## Write out the peak migration habitat proportion data
write.csv(peak, "data/sact41c1_peak_phabitat_1996-2015.csv")

## Generate Plots
ggplot(aes(x = date, y = p.10cm, color = Index, size = Index), data = peak)+
  geom_point() +
  theme_classic()
ggplot(aes(x = Index, y = p.10cm, color = Index, size = Index), data = peak)+
  geom_point() +
  theme_classic()
ggplot(aes(x = Yr.type, y = p.10cm, group = Yr.type, color = Yr.type), data = peak)+
  geom_boxplot() +
  theme_classic()
ggplot(aes(x = drought, y = p.10cm, group = drought, color = drought), data = peak)+
  geom_boxplot() +
  theme_classic()
ggplot(aes(x = drought, y = p.10cm, color = Index, size = Index), data = peak)+
  geom_point() +
  theme_classic()

####################
## Plot the proportions of <10cm habitat in SAC:T41:1 over time detected from sat imagery/topography
p <- ggplot(aes(x = year, y = p.10cm), data = peak)+
  geom_point(aes(shape = drought, color = drought), size = 3) +
  scale_color_manual(values = c("dodgerblue", "darkred"), labels = c("Non-drought", "Drought"))+
  scale_shape_manual(values = c(16, 1))+
  xlab("") + ylab("Proportion <10 cm")+
  theme_classic()+
  theme(legend.title=element_blank()) +
  theme(legend.position = "bottom")+
  theme(text = element_text(size = 12))  
png(file = "SacT41c1_phab10cm_1996-2015_drought_ndrought.png", 
    width = 76.2, height = 70, units = "mm", res = 600)
print(p2)
dev.off()  
####################

## T test to compute the difference in mean proportion habitat between drought vs. non-drought years
t.test(p.10cm ~ drought, data = peak)
#Welch Two Sample t-test

#data:  p.10cm by drought
#t = -0.18301, df = 11.93, p-value = 0.8579
#alternative hypothesis: true difference in means is not equal to 0
#95 percent confidence interval:
#  -0.05501129  0.04649092
#sample estimates:
#  mean in group 0 mean in group 1 
#0.05643017      0.06069035 

##########################################################################
## D. Frequency of optimal habitat at peak migration
##########################################################################
## Initialize empty raster stacks to store the water depth maps, and maps of <5cm 
## <10cm, and <15cm time series habitat extent maps.
depth.peak <- stack()

for(i in 1:nrow(peak)) { 
  surface <- !is.na(dem)
  maxwater <- peak$max[i]
  surface[surface==1] <- maxwater
  x <- surface - dem
  depth.peak <- stack(depth.peak, x)
  
  m <- c(-Inf, 0, 0, 0, 5, 1, 5, Inf, 0)
  peak.5cm <- reclassify(depth.peak, m)
  peak.p5cm <- cellStats(peak.5cm, 'sum')/dem.cells
  
  m3 <- c(-Inf, 0, 0, 0, 10, 1, 10, Inf, 0)
  peak.10cm <- reclassify(depth.peak, m3)
  peak.p10cm <- cellStats(peak.10cm, 'sum')/dem.cells
  
  m2 <- c(-Inf, 0, 0, 0, 15, 1, 15, Inf, 0)
  peak.15cm <- reclassify(depth.peak, m2)
  peak.p15cm <- cellStats(peak.15cm, 'sum')/dem.cells
  
  dfpeak <- data.frame(cbind(data$date.list, peak.p5cm, peak.p10cm, peak.p15cm))
}

names(dfpeak) <- c("date.list.i.", "p.5cm", "p.10cm", "p.15cm")

## Write out habitat frequency calculations and habitat frequency rasters
write.csv(dfpeak, "data/SacT41c1_peakhabitat_1996-2015.csv")
writeRaster(peak.5cm, "data/water_maps_for_test_historical_water_depth/SacT41c1_peakhab5cm_stack.tif", overwrite = T)
writeRaster(peak.10cm, "data/water_maps_for_test_historical_water_depth/SacT41c1_peakhab10cm_stack.tif", overwrite = T)
writeRaster(peak.15cm, "data/water_maps_for_test_historical_water_depth/SacT41c1_peakhab15cm_stack.tif", overwrite = T)

## Frequency maps for habitat depth ranges
peak.10cm.sum <- calc(peak.10cm, sum, na.rm=TRUE)
peak.10cm.valid <- sum(!is.na(peak.10cm))
peak.10cm.freq <- (peak.10cm.sum/peak.10cm.valid)
plot(peak.10cm.freq)
plot(poly, add = T)


#########################
## Plot the frequency of flooded habitat <10cm at SacT41c1 at the peak of migration
values(peak.10cm.freq)[values(peak.10cm.freq) == 0] <- NA

## Clip habitat frequency map to floodable region using Apr flood frequency mask
mask <- raster("data/water_maps_for_test_historical_water_depth/SacT41c1_Aprfloodfreqmask.tif")
peak.10cm.freq.clip <- overlay(peak.10cm.freq, mask, fun = function(x, y) {
  x[is.na(y[])] <- NA
  return(x)
})

plot(peak.10cm.freq.clip)

## Breaks in habitat frequency
brk = c(0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7)
p <- levelplot(peak.10cm.freq.clip, margin = FALSE, col.regions = colorRampPalette(brewer.pal(9, "Spectral"))(18), at = brk, 
              scales=list(x=list(cex = 1.5, rot=90), y=list(cex = 1.5)),
              #colorkey=list(labels=list(cex=1, font=1),title=expression(Depth (cm))))
              colorkey=list(labels=list(cex=1.5, font=1))) #+

## Save figure
png(file = "SacT41c1_freq_10cm_1996-2015.png", 
    width = 190, height = 190, units = "mm", res = 600)
print(p)
dev.off()

#################################################################################
### E. Plot Landsat-derived flood frequency from Landsat at the peak of migration
#################################################################################
## Water frequency maps for each month were generated for analysis by Schaffer-Smith et al. (2017)
## Aprilall_freq.tif summarizes the number of times water was detected in each pixel in April.
floodfreqApr <- raster("data/Aprilall_freq.tif") 
floodfreqApr <- spatial_sync_raster(floodfreqApr, dem, method="ngb")
values(floodfreqApr)[values(floodfreqApr) == 0] <- NA
plot(floodfreqApr)

## Save the flood frequency raster to use as a mask. 
## Clip flood frequency map to floodable region using dem
floodfreqApr.clip <- overlay(floodfreqApr, dem, fun = function(x, y) {
  x[is.na(y[])] <- NA
  return(x)
})
plot(floodfreqApr.clip)

## Remap the ranges to set areas that never flood to NA
m <- c(-1, 0, NA)
mask <- reclassify(floodfreqApr.clip, m)
writeRaster(mask, "data/water_maps_for_test_historical_water_depth/SacT41c1_Aprfloodfreqmask.tif", overwrite = T)

########################################################################################
## Plot flood frequency detected from Landsat at SacT41c1 at peak migration (April)
########################################################################################
## Set breaks for plotting
brk = c(0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7)
p <- levelplot(floodfreqApr.clip, margin = FALSE, col.regions = colorRampPalette(brewer.pal(9, "Spectral"))(18), at = brk, 
               scales=list(x=list(cex = 1.5, rot=90), y=list(cex = 1.5)),
               colorkey=list(labels=list(cex=1.5, font=1))) #+

png(file = "SacT41c1_floodfreq1984-2015.png", 
    width = 190, height = 190, units = "mm", res = 600)
print(p3)
dev.off()


